/**
 * 
 */
package pt.digitalis.sampleApp.iss;

import pt.digitalis.iss.IProcess;
import pt.digitalis.iss.ProcessResults;

/**
 * Abstract class that provides a base implementation of a WritingProcess for
 * performance testing. This simply dumps a sequence of strings into a provided
 * container. The container methods are only implemented in this base class
 * implementations.
 * 
 * @author Pedro Viegas <a
 *         href="mailto:pviegas@digitalis.pt">pviegas@digitalis.pt</a>
 * @created Aug 5, 2007
 * 
 */
public abstract class BaseWriterProcess implements IProcess {

	/** number of strings to output */
	private int totalInteractions;

	/**
	 * the ID that will be added to the output for identification process
	 */
	protected int processID;

	/**
	 * The processing end time
	 */
	private long endTime;

	/**
	 * Initializes the required parameters
	 * 
	 * @param totalInteractions
	 *            number of strings to output
	 * @param processID
	 *            the ID that will be added to the output for identification
	 *            process
	 */
	public void init(int totalInteractions, int processID) {
		this.totalInteractions = totalInteractions;
		this.processID = processID;
	}

	/**
	 * @see pt.digitalis.iss.IProcess#execute()
	 */
	public ProcessResults execute() {

		int i = 0;

		if (StartWritingProcess()) {

			// Do the operation itself...

			for (i = 0; i < totalInteractions; i++) {
				Write("[Process ID:"
						+ String.format("0009", processID)
						+ "] Outputing some unimportant testing string to the container: Interaction #"
						+ i);
			}

			EndWritingProcess();
		}

		ProcessResults result = new ProcessResults();

		result.setResult("lines", Long.valueOf(i));

		endTime = System.currentTimeMillis();

		// The total lines added
		return result;
	}

	/**
	 * Perform initialization operations
	 * 
	 * @return true if all initialization operations were completed sucessfullly
	 */
	abstract protected boolean StartWritingProcess();

	/**
	 * Writes a line to the container
	 * 
	 * @param item
	 *            line to write to the container
	 */
	abstract protected void Write(String item);

	/**
	 * Perform wrapup operations
	 */
	abstract protected void EndWritingProcess();

	/**
	 * @return the end time
	 */
	public long getEndTime() {
		return endTime;
	}
}
