/** 2007, Digitalis Informatica. All rights reserved.
 * 
 * Distribuicao e Gestao de Informatica, Lda.
 * Estrada de Paco de Arcos num.9 - Piso -1
 * 2780-666 Paco de Arcos
 * Telefone: (351) 21 4408990
 * Fax: (351) 21 4408999
 * http://www.digitalis.pt 
 */

package pt.digitalis.sampleApp.iss;

import pt.digitalis.iss.IProcess;
import pt.digitalis.iss.ISS;
import pt.digitalis.iss.ProcessResults;
import pt.digitalis.iss.ProcessState;
import pt.digitalis.iss.ServiceRequest;

/**
 * This class simulates parallel requests being made to the AppServer. The
 * attributes include the ISS who will execute the request, a user-supplied id
 * and the service request that will be executed.
 * 
 * @author Rodrigo Gonçalves <a
 *         href="mailto:rgoncalves@digitalis.pt">rgoncalves@digitalis.pt</a><br/>
 */
public class myProcessCaller extends Thread {

	/** The ISS that will serve the request. */
	private ISS theISS;

	/** An id. */
	String id = "";

	/** The request */
	ServiceRequest req;

	/** The process to delegate. */
	IProcess theProcess;

	/** The results after execution */
	ProcessResults results;

	/**
	 * Ctor.
	 * 
	 * @param theProcess
	 *            the process to execute
	 * @param theISS
	 *            the ISS that should execute the process
	 * @param id
	 *            this Caller's id
	 */
	public myProcessCaller(IProcess theProcess, ISS theISS, String id) {
		super("myProcessCaller-" + id);
		this.theProcess = theProcess;
		this.theISS = theISS;
		this.id = id;
	}

	/**
	 * Inspector for the service request attribute.
	 * 
	 * @return the service request
	 */
	public ServiceRequest getRequest() {
		return this.req;
	}

	/**
	 * Overriding Thread#run()...
	 * 
	 * @see java.lang.Thread#run()
	 */
	public void run() {
		try {
			if (theISS == null) {

				// No ISS execute immediately
				results = theProcess.execute();

			} else {

				// Delegate execution to the ISS manager
				this.req = theISS.delegate(this.theProcess, this.id);

				while (req.getState() != ProcessState.FINISHED
						&& req.getState() != ProcessState.ERROR) {
					try {
						synchronized (this) {
							// ...go to sleep
							wait(500);
						}
					} catch (InterruptedException e) {
						e.printStackTrace();
					}

					req = theISS.fetchResults(id);
				}

				results = new ProcessResults();
				results.setResults(req.getResults());
			}

		} catch (OutOfMemoryError e) {
			System.out
					.println("Process #"
							+ id
							+ ": Out of memory! Releasing failed process objects from RAM...");

			req = null;
			results = null;
		}
	}

	/**
	 * @return the results
	 */
	public ProcessResults getResults() {
		return results;
	}

	/**
	 * @return the theProcess
	 */
	public IProcess getTheProcess() {
		return theProcess;
	}
}
