/**
 * 
 */
package pt.digitalis.sampleApp.util;


/**
 * @author Pedro Viegas <a
 *         href="mailto:pviegas@digitalis.pt">pviegas@digitalis.pt</a>
 * @created Aug 8, 2007
 * 
 */
public class Utilities {

	/**
	 * Return the given memory value as a String
	 * 
	 * @param memory
	 *            value to translate
	 * @return the memory as a string with measurement
	 */
	static private String memoryToString(long memory) {
		if (memory > 2048)
			return (memory / 1024 + "Mb");
		else
			return memory + "kb";
	}

	/**
	 * Logs the current memory usage
	 * 
	 * @return the message with the memory info
	 */
	static public String logMemoryMessage() {

		long total = Runtime.getRuntime().totalMemory() / 1024;
		long free = Runtime.getRuntime().freeMemory() / 1024;

		return "Memory: " + ((free * 100) / total) + "% free ["
				+ memoryToString(free) + " of " + memoryToString(total) + "]";

	}

	/**
	 * Formats the given string in a number of fixed characters
	 * 
	 * @param str
	 *            the string to format
	 * @param size
	 *            the size to obtain
	 * @return the formatted string
	 */
	static public String fillRight(String str, int size) {
		return fillRight(str, size, " ");
	}

	/**
	 * Formats the given string in a number of fixed characters
	 * 
	 * @param str
	 *            the string to format
	 * @param size
	 *            the size to obtain
	 * @param fillPattern
	 *            the pattern string to use for filling
	 * @return the formatted string
	 */
	static public String fillRight(String str, int size, String fillPattern) {
		while (str.length() <= size)
			str += fillPattern;

		return str.substring(0, size);
	}

	/**
	 * Formats the given string in a number of fixed characters
	 * 
	 * @param str
	 *            the string to format
	 * @param size
	 *            the size to obtain
	 * @return the formatted string
	 */
	static public String fillLeft(String str, int size) {
		return fillLeft(str, size, " ");
	}

	/**
	 * Formats the given string in a number of fixed characters
	 * 
	 * @param str
	 *            the string to format
	 * @param size
	 *            the size to obtain
	 * @param fillPattern
	 *            the pattern string to use for filling
	 * @return the formatted string
	 */
	static public String fillLeft(String str, int size, String fillPattern) {
		while (str.length() <= size)
			str = fillPattern + str;

		return str.substring(str.length() - size);
	}
}
